import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.IOException; 
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.ArrayList;
/**
 * Reads an input text file 
 * @author Anthony Hoang
 * @version 3.0
 */
public class SystemMessage extends Actor
{
    private float FONT_SIZE;
    private int WIDTH;
    private int HEIGHT;
    private String curString;
    
    private int index;
    private int pos;
    private int hPos;
    
    private String nxtFile;
    private String curFile;
    private boolean closing;
    
    ArrayList<String> words;
    public GreenfootImage image;
    private int actsLeft;

    public SystemMessage() {}
    public SystemMessage(String text)
    {
        initialize(text);
    }
    public void initialize(String text)
    {
        closing = false;
        words = new ArrayList<String>();
        
        curFile = text;
        FONT_SIZE = 25f;
        WIDTH = 600;
        String temp = "data/text/"+text;
        if (!temp.endsWith(".txt")) //error check
        temp+= ".txt";
        curString = getTextFile(temp);
        actsLeft = curString.length()*3;
        HEIGHT = calcHeight(curString);
        readStringToArray(curString);
        
        index = 0;
        pos = (int)(FONT_SIZE);
        hPos = (int)(FONT_SIZE*1.3);
        
        image = new GreenfootImage(WIDTH, HEIGHT);
        image.setColor(new Color(255,255,255, 128));
        image.fillRect(0, 0, WIDTH, HEIGHT);
        image.setColor(new Color(0, 0, 0, 128));
        image.fillRect(3, 3, WIDTH-6, HEIGHT-6);
        Font font;
        font = new Font("Lucida Console",getImage().getFont().getStyle(), getImage().getFont().getSize());
        if (!font.getFontName().equals("Lucida Console"))
        font = new Font("Monospaced",getImage().getFont().getStyle(), getImage().getFont().getSize());
        font = font.deriveFont(FONT_SIZE);
        image.setFont(font);
        image.setColor(Color.WHITE);
        setImage(image);
    }
    public void act()
    {
        if (closing)
        {
            int t = getImage().getTransparency()-20;
            if (t < 20)
            getWorld().removeObject(this);
            else
            getImage().setTransparency(t);
            return;
        }
        actsLeft--;
        if (actsLeft < 0) closing = true;
        String key = Greenfoot.getKey();
        ///****
        if (words.size() == 0 && (Greenfoot.mouseClicked(this) || (key!=null  ) ))
        {
            if (nxtFile != null && !curFile.equals(nxtFile))
            {
                initialize(nxtFile);
            }
            else
            closing = true;
        }
        if (words.size() == 0) 
            return;
        else
        if (words.get(0).equals("@@"))
        {
            words.remove(0);
            nxtFile = words.remove(0);
        }
        else if (pos > WIDTH-FONT_SIZE*words.get(0).length() && index == 0 )
        {
            pos = (int)FONT_SIZE;
            hPos += (int)FONT_SIZE*2-FONT_SIZE/1.5;
        }
        else
        {
            String temp = words.get(0).substring(index, index + 1);
            image.drawString(temp, pos, hPos);
            index += temp.length();
            pos += (int)FONT_SIZE/2;
            setImage(image);
            
            if (index >= words.get(0).length())
            {
                words.remove(0);
                index = 0;
                pos+= FONT_SIZE/2;
            }
        }
    }
    public String getTextFile(String textFileName)
    {
        String str = "";
        StringBuffer sb = new StringBuffer();
        try{
            URL url = getClass().getClassLoader().getResource(textFileName);
            if(url == null)
            throw new IOException("File not found: " + textFileName);
            InputStream is = url.openStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(is));
            String line = br.readLine();
            while(line != null)
            {
                if (line != null)
                str += " "+line;
                line = br.readLine();
            }
            is.close();
            br.close();
        }
        catch(Exception ex)
        { return str; }
        return str;
    } 
    public void readStringToArray(String str)
    {
        StringTokenizer st = new StringTokenizer(str);
        while (st.hasMoreTokens())
        words.add(st.nextToken());
    }
    
    public int calcHeight(String str)
    {
        int numRow = str.length()*(int)FONT_SIZE/WIDTH;
        if (numRow == 0)
        numRow = 1;
        //return numRow*(int)FONT_SIZE;
        return numRow*(int)FONT_SIZE+(int)FONT_SIZE;
    }
    public Midori getMWorld()
    {
        return (Midori) getWorld();
    }
}